$TITLE  A STATIC MULTI-REGIN MODEL OF ELECTRICITY PRODUCTION AND TRADE

* READ IN SCALE OF THE MODEL: FULLSCALE OR 2-REGION EXAMPLE ----------------------------------------------

$if not set nash      $set nash no

*       Elasticities of industrial demand:

$if not set sigmaibase  $set sigmaibase 1.2
$if not set sigmaipeak  $set sigmaipeak 1.01

*       Elasticities of residential demand:

$if not set sigmarbase  $set sigmarbase 1.01
$if not set sigmarpeak  $set sigmarpeak 1.01

$set fullscale yes

$ontext

This version of the model introduces two categories of demand: those
customers who face differentiated pricing over the load-curve (i.e.
industrial customers) and those who pay a flat fee for electricity
delivered at any point on the load curve (i.e. residential customers).
$offtext


SET
        f               Firm types,
        r               Regions in the network,
        i               Generation technologies,
        l               Load configurations,
        df(f)           Dominant firms (case: imperfect competition),
        ren(i)          Renewable generation technologies;

ALIAS (r,rr,rs,rd), (i,j), (f,ff), (l,ll);


SET     b(r,rr)        	        Adjacency indicator for regions with transmission constraints,
        p(i,f,r)                Logical flag that firm f owns a plant of tech i in region r,
        route(rs,rd,r,rr)       Mapping from trade flows to borders;

PARAMETER
        xcap(i,f,r)     Upper bound on generating capacity,
        tcap(r,rr)      Transmission capacities,
        c(i,r,l)        Variable cost of technology i in region r and load l,
        loss(r,rr)      Transmission loss percentage,
        pe0(r,l)        Reference industrial demand price,
        per0(r)        	Reference residential demand price,
        di0(r,l)        Reference industrial demand,
        dr0(r,l)        Reference residential demand,
        sigmai(r,l)     Elasticity of industrial demand,
        sigmar(r,l)     Elasticity of residential demand,
        fee(r,rr,l)     Transmission fees in excess of congestion charges,
        pk(i,f,r)       Capital installation prices,
        duration(l)     Number of hours in load curve,
        rm(r,l)        	Reserve margin,
        fc(f)           Fixed cost by firm type,
        gc(r)           Grid charges,
        co2coef(i)      Emission (CO2) coefficients for technologies [ton of CO2 per GWh]
        co2_limit(r)    Regional emission limit
        rq(r)           Renewable quota (as share of domestic renewable energy supply in domestic energy demand),
        feedin_adj(i)   Adjustment factor to account for technology specific differences in feed-in tariffs;


*       Flags for scenario settings

SCALAR          nash            Flag for imperfect competitive markets /0/,
                feed            Flag for feed-in subsidies /0/,
                tquota          Flag for tradable green quota /0/,
                rquota          Flag for regional green quota /0/;

$if %fullscale%==no $goto smallscale

* FULLSCALE PARAMETERIZATION ------------------------------------------------------------------------------

PARAMETER       rq_init(r)      Renewable quota (as share of domestic renewable energy supply in domestic energy demand);

SET             re15(r)        	EU-15 Regions;

$GDXIN input.gdx
$LOAD   r=re  re15=re15  f=afi  df=df  i=tec_all  ren=tec_res  l=l  p=p  b=b  route=route
$LOAD   xcap=xcap  tcap=tcap  rm=rm  gc=gc  c=c  loss=loss  pe0=pe0  per0=per0  di0=di0        dr0=dr0
$LOAD   fee=fee co2coef=ecoef rq_init=rese_sh
$GDXIN

*       Data initialisation
co2_limit(r)    = 1e+12;
co2coef(i)      = co2coef(i)/1e+6;
sigmai(r,"base") = %sigmaibase%;
sigmar(r,"base") = %sigmarbase%;
sigmai(r,"peak") = %sigmaipeak%;
sigmar(r,"peak") = %sigmarpeak%;
pk(i,f,r)       = 1;
fc(f)           = 0;
rq(r)           = 0;

duration("base") = 7000;
duration("peak") = 700;

PARAMETER feedin_adj_init(i)    Magnitude of technology diversification (wind onshore = 1)
        /
        R_FSL_1         1.4423, R_FSL_2  1.4423, R_GEO_1  1.5884, R_GEO_2  1.5884, R_LFG_1  1.4423, R_LFG_2  1.4423,
        R_LHY_1  0.8652, R_LHY_2  0.8652, R_LHY_3  0.8652, R_LWA_1  1.4423, R_LWA_2  1.4423, R_MHY_1  0.8652,
        R_MHY_2  0.8652, R_MHY_3  0.8652, R_MSW_1  1.4423, R_MSW_2  1.4423, R_PHV_1  6.1183, R_PHV_2  6.1183,
        R_PHV_3  6.1183, R_SFU_1  1.4423, R_SFU_2  1.4423, R_SHY_1  0.8652, R_SHY_2  0.8652, R_SHY_3  0.8652,
        R_SSL_1  1.0085, R_SSL_2  1.0085, R_STH_1  6.1183, R_STH_2  6.1183, R_STH_3  6.1183, R_SWA_1  1.4423,
        R_SWA_2  1.4423, R_TID_1  0.8652, R_TID_2  0.8652, R_WAV_1  0.8652, R_WAV_2  0.8652, R_WOF_1  1.0775,
        R_WOF_2  1.0775, R_WOF_3  1.0775, R_WOF_4  1.0775, R_WON_1  1.0000, R_WON_2  1.0000, R_WON_3  1.0000,
        R_WON_4  1.0000, W1       0.8652, W2       0.8652, RES      1.4423
        /;

feedin_adj_init(i) = feedin_adj_init(i) / feedin_adj_init("R_FSL_1");

SET     reptec  Technology aggregate for reporting
        /
        NUC, HYD,  SCO,  HCO, NGS, OIL, BIO, WND, SOL, OTH
        /

ALIAS (reptec,reptec_);
$INCLUDE repmap.inc

$goto start

*       --------------------------------------------------------------------------------------------------

$label smallscale

* SMALL-SCALE PARAMETERIZATION ---------------------------------------------------------------------------

*       Two region example:
SET
        f                 / f1,f2 /,
        r                 / r1,r2 /,
        re15(r)        	  / r1,r2 /,
        i                 / hydro, therm, solar  /,
        l                 / base, peak /;
df(f) = yes;

SET
        b(r,rr)        	  / r1.r2 /
        p(i,f,r)          / hydro.f1.r1, therm.f1.r1, solar.f1.r1, hydro.f2.r2, therm.f2.r2, solar.f2.r2  /,
        route(rs,rd,r,rr) / r1.r2.r1.r2, r2.r1.r2.r1/,
        ren(i)            / hydro, solar/
        resmap(ren,ren);


b(r,rr)$b(rr,r) = yes; df(f) = yes;

PARAMETER feedin_adj_init;

feedin_adj_init("hydro") = 1;
feedin_adj_init("solar") = 2.5;

*       Data is currently randomly specified:

*.xcap(p)               = 0.5;
xcap("hydro","f1","r1") = 0.1;
xcap("therm","f1","r1") = 0.85;
xcap("solar","f1","r1") = 0.05;

xcap("hydro","f2","r2") = 0.3;
xcap("therm","f2","r2") = 0.5;
xcap("solar","f2","r2") = 0.2;

xcap("therm",f,r) = 1.7 * xcap("therm",f,r);

*.tcap(b)    = 0.02;
tcap(b)             = 0.2;
c(i,r,l)     = ord(i);
loss(r,rr)   = 0.02;
pe0(r,l)     = ord(l);
per0(r)             = 1 / card(l);
*.di0(r,l)   = ord(l) * ord(r)**0.25;
*.dr0(r,l)   = ord(l) * ord(r)**0.25;
di0(r,l)   = ord(l)**0.25;
dr0(r,l)   = ord(l)**0.25;
*sigmai(r,l)  = 8*card(l) /ord(l);
*sigmar(r,l)  = 2*card(l) /ord(l);
sigmai(r,l)  = 1.001;
sigmar(r,l)  = 1.001;
fee(r,rr,l)  = 0;
pk(i,f,r)    = 1;
rm(r,l)             = 0.06;
fc(f)        = 0;
duration(l)  = 1 / ord(l);
gc(r)        = 0;
co2coef(i)   = 0.001;
co2_limit(r) = 1e+12;
rq(r)        = 0;

*       --------------------------------------------------------------------------------------------------

$label start


* MODEL SPECIFICATION  -----------------------------------------------------------------------------------

*       Declare variables, equations and model:

POSITIVE VARIABLES
        W(f,r,l)        Marginal cost by firm - region - load period,
        THETAI(f,r,l)   Firm shares in industrial markets,
        THETAR(f,r)     Firm shares in residential markets,
        THETAL(r,l)     Load shares in residential markets,
        SI(f,r,l)       Industrial sales by firm - region - load period,
        SR(f,r)         Residential sales by firm,
        X(i,f,r,l)      Supply by firm - technology - region - load period,
        E(f,r,rr,l)     Electricity exports by firm - source - destination - load period,
        PE(r,l)         Industrial electricity prices by region - load period,
        PER(r)          Residential electricity prices,
        TAU(r,rr,l)     Shadow price on transmission capacity between adjacent regions,
        MU(i,f,r)       Shadow price on capacity,
        K(i,f,r)        Generating capacity,
        PI(r,l)         Shadow value on reserve capacity,
        Z(i,f,r,l)      Set-aside level,
        N(f)            Number of firms in equilibrium,
        PCO2(r)         Shadow price of CO2 emission constraint,
        lambda(r)       Shadow price on green quota,
        psi(r)          Ad-valorem tax on electricity sales to finance feed-in tariffs,
        qexp(r)         Exports of green quotas,
        qimp(r)         Imports of green quotas
        qpfx            Price of tradable quota,
        BETAI(r,l),
        BETAR(r,l);


EQUATIONS
        reserve, setaside, flowcap, profit, output, return, supply, imarket,
        rmarket, lshare, ishare, rshare, entry, ipricing, rpricing, trade,
        emissions, ren_quota, ele_tax, q_trdbal, q_exports, q_imports, cost_tax,
        bmk_demandi, bmk_demandr;

MODEL   electrade
        /
        reserve.PI,     setaside.Z,     flowcap.TAU,    profit.X,       output.MU,
        return.K,       supply.W,       imarket.PE,     rmarket.PER,    lshare.thetaL,
        ishare.THETAI,  rshare.THETAR,         entry.N,        ipricing.SI,     rpricing.SR,
        trade.E,        emissions.PCO2, ren_quota.lambda, ele_tax.psi,         q_exports.qexp,
        q_imports.qimp, q_trdbal.qpfx
        bmk_demandi.BETAI, bmk_demandr.BETAR
        /;

*       --------------------------------------------------------------------------------------------------



* ZERO PROFIT FOR PRODUCTION, CAPACITY CONSTRAINTS, MARKET CLEARANCE -------------------------------------

*       Operating cost of a technology must cover the marginal  value of supply:

profit(i,f,r,l)$p(i,f,r)..      c(i,r,l) + MU(i,f,r) + PCO2(r)*co2coef(i)  - (feedin_adj(i)*lambda(r))$(ren(i) and feed)
                                                                                + (rq(r)*lambda(r))$(tquota or rquota)
                                         =g= W(f,r,l)  +  lambda(r)$(ren(i) and (tquota or rquota));

*       Generation capacity:

output(i,f,r)$p(i,f,r)..        K(i,f,r) =g= sum(l, X(i,f,r,l) + Z(i,f,r,l));


*       Aggregate supply:

supply(f,r,l)..        	        sum(p(i,f,r), X(i,f,r,l)) + sum(rr, E(f,rr,r,l)*(1-loss(rr,r)) - E(f,r,rr,l))
                                         =e= SI(f,r,l) + THETAL(r,l) * SR(f,r);

*       Constant elasticity demand:

imarket(r,l)..         sum(f, SI(f,r,l)) =e= di0(r,l)*((PE(r,l)*(1+psi(r)))/pe0(r,l))**(-sigmai(r,l));

rmarket(r)..           sum(f, SR(f,r))   =e= sum(l, dr0(r,l) * ((PER(r)*(1+psi(r)))/per0(r))**(-sigmar(r,l)));


* COURNOT PRICING BEHAVIOUR ------------------------------------------------------------------------------

*       Pricing of supply to the industrial market:

ipricing(f,r,l)..       W(f,r,l) + gc(r) + rm(r,l)*PI(r,l)  - BETAI(r,l)
                                =G= PE(r,l) * (1 - (THETAI(f,r,l)/sigmai(r,l))$(df(f) and NASH));

rpricing(f,r)..        	sum(l, THETAL(r,l)*(W(f,r,l) + rm(r,l)*PI(r,l) - BETAR(r,l)))  + gc(r)
                                =G= PER(r) * (1 - (THETAR(f,r)*sum(l,THETAL(r,l)/sigmar(r,l)))$(df(f) and NASH));


* MARKET SHARES AND LOAD SHARES --------------------------------------------------------------------------

*       Load curve shares in residential markets:

lshare(r,l)..           THETAL(r,l) * sum(ll, dr0(r,ll) * (PER(r)*(1+psi(r))/per0(r))**(-sigmar(r,ll)))
                                =e= dr0(r,l) * (PER(r)*(1+psi(r))/per0(r))**(-sigmar(r,l));

*       Market shares for an individual firm in industrial markets:

ishare(f,r,l)..        	THETAI(f,r,l) * sum(ff, SI(ff,r,l)) * N(f) =e= SI(f,r,l);

*       Market shares for an individual firm in residential markets:

rshare(f,r)..           THETAR(f,r) * sum(ff, SR(ff,r)) * N(f) =e= SR(f,r);


* TRADE --------------------------------------------------------------------------------------------------

*       The flow across a border from r to rr is bounded by the transmission capacity:

flowcap(r,rr,l)$b(r,rr)..       tcap(r,rr) =g=   sum((f,rs,rd)$route(rs,rd,r,rr), E(f,rs,rd,l))
                                               - sum((f,rs,rd)$route(rs,rd,rr,r), E(f,rs,rd,l));

*       Transmission price includes exogenous transmission fees plus endogenous
*       shadow price on network transmission constraints:

trade(f,r,rr,l)..               W(f,r,l) + fee(r,rr,l) + sum(b, TAU(b,l)$route(r,rr,b)-TAU(b,l)$route(rr,r,b))
                                        =g= W(f,rr,l)*(1-loss(r,rr));

* MARKET ENTRY AND NUMBER OF FIRMS -----------------------------------------------------------------------

*       Rate of return:

return(i,f,r)$p(i,f,r)..        pk(i,f,r) =g= sum(l, duration(l)*MU(i,f,r));

*       Free entry condition determines the number of firms in equilibrium:

entry(f)..      fc(f) * N(f) =e=   sum((r,l), SI(f,r,l) * PE(r,l)*(1+psi(r)) * THETAI(f,r,l)/sigmai(r,l))
                                 + sum(r, SR(f,r) * PER(r)*(1+psi(r)) * sum(l, THETAL(r,l)/sigmar(r,l)));

* TECHNOLOGY CONSTRAINTS ---------------------------------------------------------------------------------

*       Reserve market requirement:

reserve(r,l)..     sum((i,f)$p(i,f,r), Z(i,f,r,l)) =g=         rm(r,l) * sum(f, SI(f,r,l) + THETAL(r,l)*SR(f,r));

*       Set-aside of capacity:

setaside(i,f,r,l)$p(i,f,r)..    MU(i,f,r) =g= PI(r,l);


*       Emission constraint:

emissions(r)..     co2_limit(r) =g= sum((i,f,l)$p(i,f,r), co2coef(i) * X(i,f,r,l));


*       Green quota (as share of renewable energy supply in total energy demand of a region)

ren_quota(r)..          sum((i,f,l)$(ren(i) and p(i,f,r)), X(i,f,r,l)) + (qimp(r)- qexp(r))$tquota
                                =g= rq(r)*sum((f,l), SI(f,r,l) + THETAL(r,l)*SR(f,r));

*       Tax on electricity sales to finance green feed-in subsidies:

ele_tax(r)..    sum(l, PE(r,l)*psi(r)*sum(f, SI(f,r,l))) + PER(r)*psi(r)*sum(f, SR(f,r))
                                =g= sum((i,f,l)$(ren(i) and p(i,f,r)), X(i,f,r,l))*lambda(r);


*       Imports of green quotas:

q_imports(r)..          qpfx      =g= lambda(r);

*       Exports of green quotas:

q_exports(r)..          lambda(r) =g= qpfx ;

*       Trade balance:

q_trdbal..         sum(r,qexp(r)) =e= sum(r, qimp(r));


* BENCHMARK -------------- ---------------------------------------------------------------------------------

*       Meet Benchmark demand (2004)

bmk_demandi(r,l)..      sum((f), SI(f,r,l))  =g= di0(r,l);

bmk_demandr(r,l)..      sum((f), THETAL(r,l)*SR(f,r))  =g= dr0(r,l);

*       -----------------------------------------------------------------------------------------------------

BETAI.fx(r,l) = 0;
BETAR.fx(r,l) = 0;

*E.FX(f,r,rr,l) = 0;

$if %fullscale%==yes $goto fulldata

* SMALL VERSION INITIALIZE LEVEL VALUES ---------------------------------------------------------------------

W.l(f,r,l) = 1;        	THETAI.l(f,r,l) = 1;            THETAR.l(f,r) = 1;      THETAL.l(r,l) = 1;
SI.l(f,r,l)= 1;         SR.l(f,r) = 1;         	        X.l(i,f,r,l) = 1;       E.l(f,r,rr,l) = 1;
PE.l(r,l)  = 1;         PER.l(r) = 1;                   PE.lo(r,l) = 0.001;     PER.lo(r)  = 0.001;
TAU.l(r,rr,l) = 1;      E.fx(f,r,r,l) = 0;              PCO2.l(r)  = 0;         lambda.l(r) = 0;
K.fx(i,f,r) = xcap(i,f,r); N.fx(f) = 1;        	        psi.l(r)    = 0;        psi.fx(r) = 0;
lambda.fx(r) = 0;       qexp.fx(r) = 0;                 qimp.fx(r) = 0;         qpfx.fx = 0;

feedin_adj(i) = 1;

$GOTO runmodel

*       -----------------------------------------------------------------------------------------------------

$label fulldata

* FULL VERSION INITIALIZE LEVEL VALUES ---------------------------------------------------------------------

W.l(f,r,l)      = 1;
PE.l(r,l)       = pe0(r,l);     PER.l(r)   = per0(r);
PE.lo(r,l)      = 0.001;        PER.lo(r)  = 0.001;
PCO2.l(r)       = 0;    lambda.l(r)     = 0;    psi.l(r)        = 0;
THETAI.l(f,r,l)        = 1;     THETAR.l(f,r)   = 1;    THETAL.l(r,l)   = 0.5;
SI.l(f,r,l)     = 1;    SR.l(f,r)       = 1;
X.l(i,f,r,l)    = 1;    Z.l(i,f,r,l)    = 0;

E.fx(f,r,r,l)   = 0;    E.l(f,r,rr,l)   = 0;    TAU.l(r,rr,l)   = 1;
E.fx(f,r,rr,l)$(not b(r,rr))  = 0;
E.fx(f,r,rr,l)$(not sum(i, p(i,f,r)))  = 0;

K.fx(i,f,r)     = xcap(i,f,r);
N.fx(f)         = 1;
psi.fx(r)       = 0;    lambda.fx(r)    = 0;
qexp.fx(r)      = 0;    qimp.fx(r)      = 0;    qpfx.fx         = 0;

feedin_adj(i) = 1;

*       ---------------------------------------------------------------------------------------------------

$LABEL runmodel

*       Do a pre-solve:

solve electrade using mcp;

$if %nash%==yes nash=1
display nash;


BETAI.fx(r,l) = BETAI.l(r,l);
BETAR.fx(r,l) = BETAR.l(r,l);


*       Report share of green energy in reference equilibrium

PARAMETER green_shr(r);

green_shr(r) = sum((i,f,l)$(ren(i) and p(i,f,r)), X.l(i,f,r,l))
                /sum((f,l), SI.l(f,r,l) + THETAL.l(r,l)*SR.l(f,r));

DISPLAY green_shr;

*       FULL VERSION Load regional green quota ------------------------------------------------------------

$if not %fullscale%==no        	rq(r)$re15(r) = rq_init(r);

*       SMALL VERSION Increase this share without internal financing of the green quota ------------

$if %fullscale%==no     rq("r1") = 1.7* green_shr("r1"); rq("r2") = 1.2 * green_shr("r2");

*       ---------------------------------------------------------------------------------------------------

DISPLAY rq;

*       Policy scenarios

SET     sc      Scenarios
        /
        BaU             Business-as-usual
        Feed            Feed-in subsdies
        AFeed           Adjusted Feed-in subsidies
        Rquota          Regional green quota
        Tquota          Tradable green quota
        /,

        bau(sc)        	/BaU/,
        sc_feed(sc)     /Feed/,
        sc_afeed(sc)    /AFeed/,
        sc_rquota(sc)   /Rquota/,
        sc_tquota(sc)   /Tquota/,

        runsc(sc)       /BaU, Feed, AFeed, Rquota, Tquota/;

*       REPORT PARAMETERS FOR THE LEVEL VALUES

PARAMETERS
        REP_W(sc,f,r,l)         REPORTING - Marginal cost by firm - region - load period,
        REP_THETAI(sc,f,r,l)    REPORTING - Firm shares in industrial markets,
        REP_THETAR(sc,f,r)      REPORTING - Firm shares in residential markets,
        REP_THETAL(sc,r,l)      REPORTING - Load shares in residential markets,
        REP_SI(sc,f,r,l)        REPORTING - Industrial sales by firm - region - load period,
        REP_SR(sc,f,r)          REPORTING - Residential sales by firm,
        REP_X(sc,i,f,r,l)       REPORTING - Supply by firm - technology - region - load period,
        REP_E(sc,f,r,rr,l)      REPORTING - Electricity exports by firm - source - destination - load period,
        REP_PE(sc,r,l)          REPORTING - Industrial electricity prices by region - load period,
        REP_PER(sc,r)           REPORTING - Residential electricity prices,
        REP_TAU(sc,r,rr,l)      REPORTING - Shadow price on transmission capacity between adjacent regions,
        REP_MU(sc,i,f,r)        REPORTING - Shadow price on capacity,
        REP_K(sc,i,f,r)         REPORTING - Generating capacity,
        REP_PI(sc,r,l)          REPORTING - Shadow value on reserve capacity,
        REP_Z(sc,i,f,r,l)       REPORTING - Set-aside level,
        REP_N(sc,f)             REPORTING - Number of firms in equilibrium,
        REP_PCO2(sc,r)          REPORTING - Shadow price of CO2 emission constraint,
        REP_lambda(sc,r)        REPORTING - Shadow price on green quota,
        REP_psi(sc,r)           REPORTING - Ad-valorem tax on electricity sales to finance feed-in tariffs,
        REP_qexp(sc,r)          REPORTING - Exports of green quotas,
        REP_qimp(sc,r)          REPORTING - Imports of green quotas
        REP_qpfx(sc)            REPORTING - Price of tradable quota,
        REP_BETAI(sc,r,l)       ,
        REP_BETAR(sc,r,l)       ;


*       SOME REPORTING PARAMETERS

PARAMETERS
        rep_xx          Electricity Production per Technology ,
        tech            Electricity production per AGGRAGATE Technology,
        restech        	Green Electricity production (Aggregate technologies),
        rep_p           Electricity Prices ,
        rep_ee          Cross-border Electricity Trade,
        rep_s           Regional Electricity Supply,
        rep_sf          Regional Electricity Supply,
        rep_d           Electricity Demand,
        w_diff          Difference in Marginal Value of Supply,
        green           Green shares - Quotas - Trade and Compliance,
        rep_green_val   Green value - Regional subsidy level,
        green_val_t     Total subsidies in Regions
        c_surplus       Consumer surplus,
        welfare
        sub_chk;


PARAMETER profits;


loop(sc$runsc(sc),

        psi.fx(r) = 0; lambda.fx(r) = 0; qexp.fx(r) = 0; qimp.fx(r) = 0; qpfx.fx = 0;
        feed = 0; rquota = 0; tquota = 0;
        lambda.fx(r)$(not re15(r)) = 0;
        feedin_adj(i) = 1;

        IF(bau(sc),
*       Default settings
        );

        IF(sc_feed(sc),
          feed = 1;
          psi.lo(r) = 0; psi.up(r) = + inf;
          lambda.lo(r) = 0; lambda.up(r) = +inf;
          lambda.fx(r)$(not re15(r)) = 0;
        );

        IF(sc_afeed(sc),
          feed = 1;
          feedin_adj(i) = feedin_adj_init(i);
          psi.lo(r) = 0; psi.up(r) = + inf;
          lambda.lo(r) = 0; lambda.up(r) = +inf;
          lambda.fx(r)$(not re15(r)) = 0;
        );

        IF(sc_rquota(sc),
          rquota = 1;
          lambda.lo(r) = 0; lambda.up(r) = +inf;
          lambda.fx(r)$(not re15(r)) = 0;
        );

        IF(sc_tquota(sc),
          tquota = 1;
          lambda.lo(r) = 0; lambda.up(r) = +inf;
          lambda.fx(r)$(not re15(r)) = 0;
          qexp.lo(r)   = 0; qexp.up(r)   = +inf;
          qimp.lo(r)   = 0; qimp.up(r)   = +inf;
          qpfx.lo      = 0; qpfx.up      = +inf;
          qexp.fx(r)$(not re15(r)) = 0;
          qimp.fx(r)$(not re15(r)) = 0;
        );

        solve electrade using mcp;


*       STORE REPORTING VALUES:

        REP_W(sc,f,r,l)         = W.l(f,r,l) ;
        REP_THETAI(sc,f,r,l)    = THETAI.l(f,r,l);
        REP_THETAR(sc,f,r)      = THETAR.l(f,r);
        REP_THETAL(sc,r,l)      = THETAL.l(r,l);
        REP_SI(sc,f,r,l)        = SI.l(f,r,l) ;
        REP_SR(sc,f,r)          = SR.l(f,r);
        REP_X(sc,i,f,r,l)       = X.l(i,f,r,l);
        REP_E(sc,f,r,rr,l)      = E.l(f,r,rr,l);
        REP_PE(sc,r,l)          = PE.l(r,l);
        REP_PER(sc,r)           = PER.l(r);
        REP_TAU(sc,r,rr,l)      = TAU.l(r,rr,l);
        REP_MU(sc,i,f,r)        = MU.l(i,f,r);
        REP_K(sc,i,f,r)         = K.l(i,f,r) ;
        REP_PI(sc,r,l)          = PI.l(r,l);
        REP_Z(sc,i,f,r,l)       = Z.l(i,f,r,l);
        REP_N(sc,f)             = N.l(f);
        REP_PCO2(sc,r)          = PCO2.l(r);
        REP_lambda(sc,r)        = lambda.l(r);
        REP_psi(sc,r)           = psi.l(r);
        REP_qexp(sc,r)          = qexp.l(r);
        REP_qimp(sc,r)          = qimp.l(r);
        REP_qpfx(sc)            = qpfx.l;
        REP_BETAI(sc,r,l)       = BETAI.l(r,l);
        REP_BETAR(sc,r,l)       = BETAR.l(r,l);



*       Some preliminary reporting

        rep_xx(i,f,r,l,sc)$p(i,f,r)     = X.l(i,f,r,l);

$if %fullscale%==no $goto flag
        tech(r,reptec,sc)    = ROUND(sum((p(i,f,r),l)$repmap(i,reptec), (X.l(i,f,r,l))), 3);
        restech(r,restec,sc) = ROUND(sum((p(i,f,r),l)$resmap(i,restec), X.L(i,f,r,l)), 3);
$label flag

        rep_p(r,"____","PER_T",sc)      = PER.l(r)*(1+psi.l(r));
        rep_p(r,l,"PE_T",sc)            = PE.l(r,l)*(1+psi.l(r));
        rep_p(r,"____","PER_WOT",sc)$(FEED)     = PER.l(r);
        rep_p(r,l,"PE_WOT",sc)$(FEED)           = PE.l(r,l);

        rep_ee(r,rr,sc)        	        = sum((l,f), E.l(f,r,rr,l));

        rep_s(r,sc)                     = sum((f,l), si.l(f,r,l) + thetal.l(r,l)*sr.l(f,r));
        rep_sf(r,f,sc)                  = sum((l), si.l(f,r,l) + thetal.l(r,l)*sr.l(f,r));

        w_diff(f,r,l,sc)                = W.l(f,r,l) ;

        green(r,sc,"compliance")        = (sum((i,f,l)$(ren(i) and p(i,f,r)), X.l(i,f,r,l)) + (qimp.l(r)- qexp.l(r)))
                                             / sum((f,l), SI.l(f,r,l) + THETAL.l(r,l)*SR.l(f,r));
        green(r,sc,"production")        = (sum((i,f,l)$(ren(i) and p(i,f,r)), X.l(i,f,r,l)))
                                             / sum((f,l), SI.l(f,r,l) + THETAL.l(r,l)*SR.l(f,r));
        green(r,sc,"trade")             = (qimp.l(r)- qexp.l(r))/sum((f,l), SI.l(f,r,l) + THETAL.l(r,l)*SR.l(f,r));

        rep_green_val(r,sc)             = lambda.l(r);

        green_val_t(r,sc)               = (sum((i,f,l)$(ren(i) and p(i,f,r)), X.l(i,f,r,l))
                                             + (qimp.l(r)- qexp.l(r))) * lambda.l(r) ;
        green_val_t("total",sc)        	= sum(r, green_val_t(r,sc));

        rep_d(r,"industrial",l,sc)      = di0(r,l)*((PE.l(r,l)*(1+psi.l(r)))/pe0(r,l))**(-sigmai(r,l));

        rep_d(r,"residential",l,sc)     = dr0(r,l)*((PER.l(r)*(1+psi.l(r)))/per0(r))**(-sigmar(r,l));

        rep_d(r,"total","____",sc)      = sum(l, rep_d(r,"residential",l,sc) +rep_d(r,"industrial",l,sc));

        sub_chk(r,"subsidy",sc)        	= green_val_t(r,sc) ;

        sub_chk(r,"q_financing",sc)     = (rq(r) * lambda.l(r)) * rep_s(r,sc);

        sub_chk(r,"f_financing",sc)     = sum(l, PE.l(r,l)*psi.l(r)*sum(f, SI.l(f,r,l))) + PER.l(r)*psi.l(r)*sum(f, SR.l(f,r));

);

w_diff(f,r,l,sc)$runsc(sc) = ROUND( (w_diff(f,r,l,sc) - w_diff(f,r,l,"BaU")) , 3);

parameter        prices          Electricity Prices [in EUR per MWh];

prices(r,"PER_T",sc,"____")		= 100*REP_PER(sc,r)*(1+REP_PSI(sc,r));
prices(r,"PE_T",sc,l)		= 100*REP_PE(sc,r,l)*(1+REP_PSI(sc,r));
prices(r,"PER_WOT",sc,"____")	= 100*REP_PER(sc,r);
prices(r,"PE_WOT",sc,l)		= 100*REP_PE(sc,r,l);

OPTION prices:2:2:2;

parameter        edemand         Electricity demand of HH and Ind for base and peak-load [TWh];

edemand(r,sc,"industrial",l)	= di0(r,l)*((REP_PE(sc,r,l)*(1+REP_PSI(sc,r)))/pe0(r,l))**(-sigmai(r,l));
edemand(r,sc,"residential","____")	= sum(l, dr0(r,l) * ((REP_PER(sc,r)
						*(1+REP_PSI(sc,r)))/per0(r))**(-sigmar(r,l)));
edemand(r,sc,"Total","____")	= sum(l, edemand(r,sc,"industrial",l)) + edemand(r,sc,"residential","____");

OPTION edemand:2:2:2;

parameter	subsidy         Green value - Regional subsidy level [in EUR per kWh (green value) percent (price) mill EUR (costs)];

subsidy(r,sc,"Green Value")		    = 100 * REP_LAMBDA(sc,r);
subsidy(r,sc,"Change in Prices")	    = 100 * REP_PSI(sc,r);
subsidy(r,sc,"Compliance Costs")	    = 100*(sum((i,f,l)$(ren(i) and p(i,f,r)), REP_X(sc,i,f,r,l))) 
							* subsidy(r,sc,"Green Value");

subsidy("Total",sc,"Compliance Costs")  = sum(r, subsidy(r,sc,"Compliance Costs"));
OPTION subsidy:2:1:1;

parameter        c_surplus       Consumer surplus;

*	Economic Effects - Profits, Consumer Surplus and Total Econonic Surplus


c_surplus(r,"industrial",sc,l)	=   (prices(r,"PE_T",sc,l)   - prices(r,"PE_T","BaU",l)) * 100 * edemand(r,sc,"industrial",l)
					   - ( prices(r,"PE_T",sc,l)   - prices(r,"PE_T","BaU",l)) 
						* 100 * (edemand(r,sc,"industrial",l) - edemand(r,"BaU","industrial",l))*0.5;

c_surplus(r,"residential",sc,"____") =   (prices(r,"PER_T",sc,"____") - prices(r,"PER_T","BaU","____")) 
						* 100*(edemand(r,sc,"residential","____"))
					    - (	(prices(r,"PER_T",sc,"____") - prices(r,"PER_T","BaU","____")) 
					   * 100*(edemand(r,sc,"residential","____") - edemand(r,"BaU","residential","____")))*0.5;

c_surplus(r,"TOTAL",sc,"____") 	= (c_surplus(r,"residential",sc,"____") + sum(l, c_surplus(r,"industrial",sc,l)));

c_surplus("EU","TOTAL",sc,"____")   = sum(r$re15(r), c_surplus(r,"TOTAL",sc,"____") );

OPTION c_surplus:2:2:2;

parameter        f_profits       Firm's profits;

f_profits(r,sc) =      sum(f,  sum(l, prices(r,"PE_WOT",sc,l) *  100*REP_SI(sc,f,r,l) 
					* REP_THETAI(sc,f,r,l)/sigmai(r,l)))
  			+  sum(f,  prices(r,"PER_WOT",sc,"____") 
					*  100 * sum(l, (REP_THETAL(sc,r,l)* REP_SR(sc,f,r)*REP_THETAI(sc,f,r,l)/sigmai(r,l))));

f_profits("EU",sc)			= sum(r$re15(r), f_profits(r,sc));


OPTION f_profits:3:1:1;



parameter	e_surplus       Economic Surplus;
e_surplus(r,sc)		= (c_surplus(r,"TOTAL",sc,"____") + (f_profits(r,"BaU") - f_profits(r,sc)));
e_surplus("EU",sc)		= sum(r$re15(r), e_surplus(r,sc));
OPTION e_surplus:2:1:1;

option rep_xx:4:4:1;
option rep_p:4:3:1;
option rep_ee:4:2:1;
option rep_s:3:1:1;
option rep_sf:3:1:1;
option w_diff:3:2:2;
option green:3:1:2;
display rep_x;
display rep_p;
display rep_ee;
display rep_s;
display rep_sf;
display w_diff;
display green, rep_green_val, green_val_t;
DISPLAY sub_chk;
$if %fullscale%==no $exit

option tech:3:2:1;
option restech:3:2:1;
DISPLAY tech, restech;

* 	REPLICATE TABLES IN PAPER ------------------------------------------------------------------------------------- 

set	scr	Scenarios in report /bau, feed_d, feed_h, quota_r, quota_eu /,
	scmap(sc,scr)	/bau.bau,tquota.quota_eu,rquota.quota_r,feed.feed_h,afeed.feed_d/;

*	TABLE 3

PARAMETER table3data 'Changes in electricity prices and electricity demand';

loop(scmap(sc,scr),
  table3data(scr,r,"Prices","Industrial",l)       = 100 * ((prices(r,"PE_T",sc,l) / prices(r,"PE_T","BaU",l))-1);
  table3data(scr,r,"Prices","Residential","____") = 100 * ((prices(r,"PER_T",sc,"____") / prices(r,"PER_T","BaU","____"))-1);
  table3data(scr,r,"Demand","Total","____") 	   = 100 * ((edemand(r,sc,"Total","____") / edemand(r,"BaU","Total","____")) - 1);
);
OPTION table3data:1:1:4
DISPLAY table3data;


*	TABLE 4

PARAMETER table4data 'Green values, induced electricity taxes and compliance costs';
loop(scmap(sc,scr),
  table4data(scr,r,"Green Value")	= subsidy(r,sc,"Green Value");
  table4data(scr,r,"Electricity tax")	= subsidy(r,sc,"Change in Prices");
  table4data(scr,r,"Direct costs")	= subsidy(r,sc,"Compliance Costs");
  table4data(scr,r,"Efficiency costs")	= e_surplus(r,sc);
  table4data(scr,"eu15","Direct costs")	= sum(re15(r),subsidy(r,sc,"Compliance Costs"));
  table4data(scr,"eu15","Efficiency costs") = sum(re15(r),e_surplus(r,sc));
);
OPTION table4data:1:1:2
DISPLAY table4data;

parameter        techmix         Electricity production per AGGRAGATE Technology [in TWh];

techmix(r,reptec,sc)    		= (100*sum((p(i,f,r),l)$repmap(i,reptec), (REP_X(sc,i,f,r,l))));
techmix(r,"Total",sc)    		= sum(reptec, techmix(r,reptec,sc));
techmix(r,reptec,sc)		= 100 * techmix(r,reptec,sc) / techmix(r,"Total",sc);

OPTION techmix:2:1:2;

*	TABLE 7

PARAMETER table7data 'Technology mix for electricity production';
loop(scmap(sc,scr),
  table7data(r,reptec,scr) = techmix(r,reptec,sc);
);
OPTION table7data:2:2:1
DISPLAY table7data;

$if not set inputs $exit
set inputs /%inputs%/;